#ifndef HILBERT_H
#define HILBERT_H 1
#include <fftw3.h>


using namespace std;

#ifndef PI
#define PI 3.1415926535
#endif

namespace SEPMY {
// z = x + iy, from x find y
void hilbert(float const *x, float *y, int n);

void hilbert(float const *x, float *y, int N) {
	fftw_complex real[N], imag[N], freq[N];
	// fill in source in time domain
    for (int i = 0; i < N; i++) {
    	real[i][0] =  x[i];
    	real[i][1] =  0.;
    }

    // get the imaginary part by hilbert transform
    fftw_plan forward;
    fftw_plan backward;
    forward = fftw_plan_dft_1d(N, real, freq, FFTW_FORWARD, FFTW_ESTIMATE);
    fftw_execute (forward);


    for (int i = 0; i < N; i++) {
    	if (i == 0) {
    		freq[i][0] = 0;
        	freq[i][1] = 0;
    	}
    	if (i < (N + 1) / 2) {
        	double t1 = freq[i][0];
        	double t2 = freq[i][1];
        	freq[i][0] = t2;
        	freq[i][1] = -t1;
    	}
    	if (i > N / 2) {
        	double t1 = freq[i][0];
        	double t2 = freq[i][1];
        	freq[i][0] = -t2;
        	freq[i][1] = t1;
    	}
    }

    //
    backward = fftw_plan_dft_1d (N, freq, imag, FFTW_BACKWARD, FFTW_ESTIMATE);
    fftw_execute (backward);
    for (int i = 0; i < N; i++) {
    	//std::cout << "i = " << i << ", re = " << imag[i][0] << ", im = " << imag[i][1] << std::endl;
    	y[i] = imag[i][0] / N;
    }
}
}

#endif
