#ifndef WAVECORE_H
#define WAVECORE_H 1
#include "stdio.h"
#include "math.h"
#define COEF0 -2.92722222222
#define COEF1  1.66666666667
#define COEF2 -0.23809523809
#define COEF3  0.03968253968
#define COEF4 -0.00496031746
#define COEF5  0.00031746031
#define FAT 5

namespace SEPMY {

void laplacian2D(float *model, float *data, int n1, int n2);
void laplacian3D(float *model, float *data, int n1, int n2, int n3);
void marchingKernel3D(float *prev, float *curr, float *next, float *velDtDx, int n1, int n2, int n3); // time marching
void inject(float *next, float *shotBin, int *shotGeo, int shotCount, int it, int nt);           // inject source wavelet
void setTaper(float *taper, int n1, int n2, int nb = 50, float alpha = 0.03);

void marchingKernel3D(float *prev, float *curr, float *next, float *velDtDx, int n1, int n2, int n3) {
	if (n3 > 1)
		laplacian3D(curr, next, n1, n2, n3);
	else
		laplacian2D(curr, next, n1, n2);
	for (int i = 0; i < n1 * n2 * n3; i++) {
		next[i] = 2. * curr[i] + velDtDx[i] * next[i] - prev[i];
	}
}

void inject(float *next, float *shotBin, int *shotGeo, int shotCount, int it, int nt) {
	for (int is = 0; is < shotCount; is++) {
		int loc = shotGeo[is];
		next[loc] += shotBin[is * nt + it];
	}
}


void laplacian2D(float *model, float *data, int n1, int n2) {
	float coef[11] = {COEF5, COEF4, COEF3, COEF2, COEF1, COEF0, COEF1, COEF2, COEF3, COEF4, COEF5};
	int shift1[11] = {-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5};
	int shift2[11] = {-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5};
	for (int i = 0; i < 11; i++) shift2[i] *= n1;
	float tmp;
	int loc;
	for (int i2 = FAT; i2 < n2 - FAT; i2++) {
		for (int i1 = FAT; i1 < n1 - FAT; i1++) {
			tmp = 0;
			loc = i2 * n1 + i1;
			for (int i = 0; i < 11; i++) {
				tmp += coef[i] * (model[loc + shift1[i]] + model[loc + shift2[i]]);
			}
			data[loc] = tmp;
		}
	}
}

void laplacian3D(float *model, float *data, int n1, int n2, int n3) {
	float coef[11] = {COEF5, COEF4, COEF3, COEF2, COEF1, COEF0, COEF1, COEF2, COEF3, COEF4, COEF5};
	int shift1[11] = {-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5};
	int shift2[11] = {-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5};
	int shift3[11] = {-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5};
	for (int i = 0; i < 11; i++) {
		shift2[i] *= n1;
		shift3[i] *= n1 * n2;
	}
	float tmp;
	int loc;
	for (int i3 = FAT; i3 < n3 - FAT; i3++) {
		for (int i2 = FAT; i2 < n2 - FAT; i2++) {
			for (int i1 = FAT; i1 < n1 - FAT; i1++) {
				tmp = 0;
				loc = i3 * n2 * n1 + i2 * n1 + i1;
				for (int i = 0; i < 11; i++) {
					tmp += coef[i] * (model[loc + shift1[i]] + model[loc + shift2[i]] + model[loc + shift3[i]]);
				}
				data[loc] = tmp;
			}
		}
	}
}

void setTaper(float *taper, int n1, int n2, int nb, float alpha) {

    //std::cout << "n1 = " << n1 << ", n2 = " << n2 << std::endl;
	  for(int i = 0; i < n1 * n2; i++) {
		  taper[i] = 1.0;
	  }

	  // TOP and BOTTOM
	  for (int i2 = 0; i2 < nb; i2++) {
	      for (int i1 = nb; i1 < n1 - nb; i1++) {
	          double taps = 1. * (nb - i2) / (1.0 * nb);
	          taps = (1. - alpha) + alpha * cos(PI * taps);
	          taper[i2 * n1 + i1] = taps;
	          taper[(n2 - i2 - 1) * n1 + i1] = taps;
	      }
	  }
	  // left and right
	  for (int i1 = 0; i1 < nb; i1++) {
	      for (int i2 = nb; i2 < n2 - nb; i2++){
	          double taps = 1. * (nb - i1) / (1.0 * nb);
	          taps = (1. - alpha) + alpha * cos(PI * taps);
	          taper[i2 * n1 + i1] = taps;
	          taper[i2 * n1 + n1 - i1 - 1] = taps;
	      }
	  }
	  // Corner
	  for (int i2 = 0; i2 < nb; i2++) {
	      for (int i1 = 0; i1 < nb; i1++) {
		      double taps = 1.0 * (nb - i1) * (nb - i1) + 1.0 * (nb - i2) * (nb - i2);
		      taps = sqrt(taps) / (1.0 * nb);
		      taps = (1. - alpha) + alpha * cos(PI * taps);
		      taper[i2 * n1 + i1] = taps;
		      taper[i2 * n1 + n1 - i1 - 1] = taps;
		      taper[(n2 - i2 - 1) * n1 + i1] = taps;
		      taper[(n2 - i2 - 1) * n1 + n1 - i1 - 1] = taps;
	      }
	  }
}

}
#endif
