#include "SEPMYinc.h"
#include "Vector.h"
#include "hilbert.h"
#include "../../001Green/Src/utilGreen.h"
#include <cmath>
using namespace std;
using namespace SEPMY;

/*
 *  code to generate RTM 2d image for GENESIS dataset with extended offset.
 *  VTI wave equation is applied on GPU
 *
 *  Input:
 *  	seed: int, used to shuffle shot gather, for quick debugging
 *  	offmin: float, minimum offset used for RTM
 *  	offmax: float, maximum offset used for RTM
 *
 *  Output:
 *
 *  Author: Yinbin Ma
 *  First created: 2018-01-23
 *  Last modified: 2018-01-23
 */

int main (int argc, char *argv[]) {
	initpar(argc, argv);

	// parameters related to machine
	int nt = 1500, noff = 175, ns = 1000;

	// import parameter from command line
	// set up axis
	axis ax_t(nt, 0, 0.004, "time", "ms");
	axis ax_f(noff, 0, 1, "offset", "id");
	axis ax_f2(noff * 2, -noff, 1, "offset", "id");
	axis ax_s(ns, 0, 1, "shot", "id");
	hyperCube dataCube({ax_t, ax_f, ax_s});
	hyperCube data2Cube({ax_t, ax_f2, ax_s});
	//read in data
	float *dataRecRe  = new float[nt * noff * ns];
	float *dataRecIm  = new float[nt * noff * ns];
	float *dataRecAng = new float[nt * noff * ns];
	float *dataRecAb  = new float[nt * noff * ns];
	float *dataSynRe  = new float[nt * noff * ns];
	float *dataSynIm  = new float[nt * noff * ns];
	float *dataSynAb  = new float[nt * noff * ns];
	float *dataSynAng = new float[nt * noff * ns];
	sreed("dataRec", dataRecRe, sizeof(float) * nt * noff * ns);
	sreed("dataSyn", dataSynRe, sizeof(float) * nt * noff * ns);

	// output direct comparison
	cout << "======  work on direct comparison =============" << endl;
	float *dataAll = new float[nt * noff * ns * 2];
	for (int is = 0; is < ns; is++) {
		cout <<  "work on shot " << is << ", out of " << ns << endl;
		for (int ioff = 0; ioff < noff; ioff++) {
			hilbert(&dataRecRe[is * noff * nt + ioff * nt], &dataRecIm[is * noff * nt + ioff * nt], nt);
			hilbert(&dataSynRe[is * noff * nt + ioff * nt], &dataSynIm[is * noff * nt + ioff * nt], nt);
			double energyRec = 0.;
			double energySyn = 0.;
			for (int it = 0, loc = is * noff * nt + ioff * nt; it < nt; it++, loc++) {
				double tmpRec = dataRecRe[loc] * dataRecRe[loc] + dataRecIm[loc] * dataRecIm[loc];
				double tmpSyn = dataSynRe[loc] * dataSynRe[loc] + dataSynIm[loc] * dataSynIm[loc];
				energyRec += tmpRec;
				energySyn += tmpSyn;
				dataRecAb[loc] = sqrt(tmpRec);
				dataSynAb[loc] = sqrt(tmpSyn);
				dataRecAng[loc] = dataRecAb[loc] == 0 ? 0 : asin(dataRecRe[loc] / dataRecAb[loc]);
				dataSynAng[loc] = dataSynAb[loc] == 0 ? 0 : asin(dataSynRe[loc] / dataSynAb[loc]);
			}
			double factor = sqrt(energyRec / energySyn);
			if (energySyn == 0) factor = 0;
//			if (energySyn != 0)
//			for (int it = 0, loc = is * noff * nt + ioff * nt; it < nt; it++, loc++) {
//				dataSynRe[loc] *= factor;
//				dataSynIm[loc] *= factor;
//				dataSynAb[loc] *= factor;
//			}
			for (int it = 0; it < nt; it++) {
				dataAll[is * noff * nt * 2 + ioff * nt + it] = dataRecRe[is * noff * nt + ioff * nt + it];
				dataAll[is * noff * nt * 2 + (2 * noff - ioff - 1) * nt + it] = dataSynRe[is * noff * nt + ioff * nt + it] * factor;
			}
		}

	}
	srite("dataComp", dataAll, sizeof(float) * nt * noff * ns * 2);
	data2Cube.writeAxes("dataComp");

	// work on adj data residual
	cout << "======  work on adj data =============" << endl;
	memset(dataAll, 0, sizeof(float) * nt * noff * ns * 2);
	float *tmpFirs = new float[nt];
	float *tmpDiff = new float[nt];
	float *tmpHilb = new float[nt];
	for (int is = 0; is < ns; is++) {
		cout <<  "work on shot " << is << ", out of " << ns << endl;
		for (int ioff = 0; ioff < noff; ioff++) {
			for (int it = 0, loc = is * noff * nt + ioff * nt; it < nt; it++, loc++) {
				float tmp = pow(dataRecAb[loc] / dataSynAb[loc], 2.0) * (dataRecAng[loc] - dataSynAng[loc]);
				tmpDiff[it] = dataSynAb[loc] == 0 ? 0 : tmp * dataSynRe[loc];
				tmpFirs[it] = dataSynAb[loc] == 0 ? 0 : tmp * dataSynIm[loc];
			}
			hilbert(tmpDiff, tmpHilb, nt);
			for (int it = 0, loc = is * noff * nt + ioff * nt; it < nt; it++, loc++) {
				dataSynRe[loc] = tmpFirs[it] + tmpHilb[it];
			}
			float energyRec = energy2D(&dataRecRe[is * noff * nt + ioff * nt], nt, 1);
			float energySyn = energy2D(&dataSynRe[is * noff * nt + ioff * nt], nt, 1);
			double factor = energyRec / energySyn;
			if (energySyn == 0) factor = 0;
			for (int it = 0; it < nt; it++) {
				dataAll[is * noff * nt * 2 + ioff * nt + it] = dataRecRe[is * noff * nt + ioff * nt + it];
				dataSynRe[is * noff * nt + ioff * nt + it] *= factor;
				dataAll[is * noff * nt * 2 + (2 * noff - ioff - 1) * nt + it] = dataSynRe[is * noff * nt + ioff * nt + it];
			}

		}

	}
	srite("dataCombAdj", dataAll, sizeof(float) * nt * noff * ns * 2);
	data2Cube.writeAxes("dataCombAdj");
	srite("dataAdj", dataSynRe, sizeof(float) * nt * noff * ns);
	dataCube.writeAxes("dataAdj");
//	// read in data
//	float *dataBaseRe  = new float[nt * noff];
//	float *dataBaseIm  = new float[nt * noff];
//	float *dataBaseAbs = new float[nt * noff];
//	float *dataBaseAng = new float[nt * noff];
//	float *dataMonRe   = new float[nt * noff];
//	float *dataMonIm   = new float[nt * noff];
//	float *dataMonAbs  = new float[nt * noff];
//	float *dataMonAng  = new float[nt * noff];
//	sreed("dataB", dataBaseRe, sizeof(float) * nt * noff);
//	sreed("dataM", dataMonRe, sizeof(float) * nt * noff);
//	float *tmpR = new float[nt];
//	float *tmpI = new float[nt];
//	float eps = 1000.0;
//	for (int ioff = 0; ioff < noff; ioff++) {
//		cout << "work on offset " << ioff << ", out of " << noff << endl;
//		hilbert(&dataBaseRe[ioff * nt], &dataBaseIm[ioff * nt], nt);
//		hilbert(&dataMonRe[ioff * nt], &dataMonIm[ioff * nt], nt);
//		memcpy(tmpR, &dataBaseRe[ioff * nt], sizeof(float) * nt);
//		memcpy(tmpI, &dataBaseIm[ioff * nt], sizeof(float) * nt);
//		fftHarmonic(tmpR, tmpI, &dataBaseRe[ioff * nt], &dataBaseIm[ioff * nt], nt);
//		memcpy(tmpR, &dataMonRe[ioff * nt], sizeof(float) * nt);
//		memcpy(tmpI, &dataMonIm[ioff * nt], sizeof(float) * nt);
//		fftHarmonic(tmpR, tmpI, &dataMonRe[ioff * nt], &dataMonIm[ioff * nt], nt);
//		float energyB;
//		if (ioff == 0) energyB = energy2D(&dataBaseRe[ioff * nt], nt, 1);
//		float energyM;
//		if (ioff == 0) energyM = energy2D(&dataMonRe[ioff * nt], nt, 1);
//		for (int it = 0, i = ioff * nt; it < nt; it++, i++) {
//			dataBaseAbs[i] = sqrt(dataBaseRe[i] * dataBaseRe[i] + dataBaseIm[i] * dataBaseIm[i]);
//			dataMonAbs[i]  = sqrt(dataMonRe[i] * dataMonRe[i]   + dataMonIm[i] * dataMonIm[i]);
//			dataBaseAng[i] = asin(dataBaseRe[i] / dataBaseAbs[i]);
//			dataMonAng[i]  = asin(dataMonRe[i] / dataMonAbs[i]);
//			dataBaseAng[i] *= dataMonAbs[i] * dataMonAbs[i];
//			dataMonAng[i] *= dataMonAbs[i] * dataMonAbs[i];
//		}
//	}
//	srite("angB", dataBaseAng, sizeof(float) * nt * noff);
//	dataCube.writeAxes("angB");
//	srite("angM", dataMonAng, sizeof(float) * nt * noff);
//	dataCube.writeAxes("angM");

	return 0;
}
