#include "SEPMYinc.h"
#include "Vector.h"
#include "utilGENESIS.h"
#include "PropVTI2dGPU.cu"

using namespace std;
using namespace SEPMY;

/*
 *  code to generate RTM 2d image for GENESIS dataset with extended offset.
 *  VTI wave equation is applied on GPU
 *
 *  Input:
 *  	seed: int, used to shuffle shot gather, for quick debugging
 *  	offmin: float, minimum offset used for RTM
 *  	offmax: float, maximum offset used for RTM
 *
 *  Output:
 *
 *  Author: Yinbin Ma
 *  First created: 2018-01-23
 *  Last modified: 2018-01-23
 */

int main (int argc, char *argv[]) {
	initpar(argc, argv);
	int nt = 2000, noff1 = 175, noff2 = 175, noff = noff1 + noff2, nm = 1000;
	int shift = 20;
	float *d1 = new float[nt * noff1 * nm];
	float *d2 = new float[nt * noff2 * nm];
	float *d  = new float[noff * (2 * shift + 1) * nm];
	memset(d, 0, sizeof(float) * noff1 * (2 * shift + 1) * nm);
	sreed("data1", d1, sizeof(float) * nt * noff1 * nm);
	sreed("data2", d2, sizeof(float) * nt * noff2 * nm);

	// compute cross correlation
	for (int im = 0; im < nm; im++) {
		cout << "im = " << im << endl;
		int shiftD1 = im * nt * noff1;
		int shiftD2 = shiftD1;
		int shiftC  = im * noff * (2 * shift + 1);		
		for (int ishift = -shift, id = 0; ishift <= shift; ishift++, id++) {
			//cout << "ishift = " << ishift << endl;
			for (int ioff = 0; ioff < noff1; ioff++) {
				//cout << "	ioff = " << ioff << endl;
				for (int it = max(ishift, 0); it < min(nt + ishift, nt); it++) {
					d[id * noff1 + ioff + shiftC] +=	d1[ioff * nt + it + shiftD1] * d2[ioff * nt + it - ishift + shiftD1];
				}
			}
		}
	}
	// normalize
	for (int im = 0; im < nm; im++) {
		int shiftC  = im * noff * (2 * shift + 1);	
		for (int ioff = 0; ioff < noff1; ioff++) {
			float maxv = 0;
			for (int id = 0; id < 2 * shift + 1; id++) {
				maxv = max(maxv, d[id * noff1 + ioff + shiftC]);
			}
			for (int id = 0; id < 2 * shift + 1; id++) {
				d[id * noff1 + ioff + shiftC] = d[id * noff1 + ioff + shiftC] / maxv;
			}
		}
	}
	srite("data", d, sizeof(float) * noff1 * (2 * shift + 1)  * nm);
	return 0;
}
