#include "SEPMYinc.h"
#include "Vector.h"
#include "utilGENESIS.h"
#include "randomBound.cu"
#include "PropVTI2dGPU.cu"

using namespace std;
using namespace SEPMY;

/*
 *  code to generate RTM 2d image for GENESIS dataset with extended offset.
 *  VTI wave equation is applied on GPU
 *
 *  Input:
 *  	seed: int, used to shuffle shot gather, for quick debugging
 *  	offmin: float, minimum offset used for RTM
 *  	offmax: float, maximum offset used for RTM
 *
 *  Output:
 *
 *  Author: Yinbin Ma
 *  First created: 2018-01-23
 *  Last modified: 2018-01-23
 */

int main (int argc, char *argv[]) {
	initpar(argc, argv);



	int ns, nr, nt, nz, nx, nhx = 1;
	float ot, oz, ox, dt, dz, dx;
	hyperCube realmodelCube, dataCube, geomCube;
	realmodelCube.readAxes("slow");
	hyperCube modelCube = expandModelCube(realmodelCube);
	extractPar(&modelCube, nx, ox, dx, 1);
	extractPar(&modelCube, nz, oz, dz, 2);
	axis ax_s(1000, 0, 1, "label", "label");
	// physical parameters for RTM
    float vmax = 5.0;
    float vmin = 1.5;
    float fmax = 50;
    float dtProp = 0.8;
    cout << "CFL        = " << vmax * dtProp / dx << endl;
    cout << "Dispersion = " << vmin * 1000 / fmax / dz << endl;

	// debug mode if necessary

    // Load wavelet, data and model
    float *source = new float[nt * 2 + 1];
    //sreed("time", source, sizeof(float) * (nt * 2 + 1));
	for (int it = 0; it < nt * 2 + 1; it++) {
		int x = it - nt;
		if (x == 0)
			source[it] = 1.;
		else {
			float wx = x * dt * 0.001 * 400.;
			source[it] = sin(wx) / wx;
		}
	}
//	srite("test", source, sizeof(float) * (2 * nt + 1));
//	axis ax_t(nt * 2 + 1, -dt * nt * 0.001, dt * 0.001, "shot", "sx id");
//	hyperCube tmpCube({ax_t});
//	tmpCube.writeAxes("test");
//	return 0;
    int modelSizeSmall = realmodelCube.getSize();
    int modelSize = modelCube.getSize();
    float *velTmp   = new float[modelSizeSmall];
    float *vel      = new float[modelSize];
    sreed("slow", velTmp, sizeof(float) * modelSizeSmall);
    for (int i = 0; i < modelSizeSmall; i++) {
    	velTmp[i] = 1.0 / velTmp[i];
    }
    fillModel(realmodelCube, modelCube, velTmp, vel);
  	float * savedRandomV = new float[2136 * 792 * 1000];


    for (int is = 0; is < 100; is++) {
    	cout << "is = " << is << endl;
        vector<int> dims = {nx, nz, 1};
        vector<int> thick = {50, 50, 1};
        float pctG = 0.9;
        int seed = is * 27817;
        memcpy(&savedRandomV[is * nx * nz], vel, sizeof(float) * modelSize);
        RandomMY(&savedRandomV[is * nx * nz], 1, 0.2, dims, thick, pctG, seed);
    }
    srite("velRand", savedRandomV, sizeof(float) * nx * nz * 100);
    modelCube.writeAxes("velRand");
	return 0;
}
