#include "SEPMYinc.h"
#include "Vector.h"
#include "hilbert.h"
#include "../../001Green/Src/utilGreen.h"
#include <cmath>
using namespace std;
using namespace SEPMY;

/*
 *  code to generate RTM 2d image for GENESIS dataset with extended offset.
 *  VTI wave equation is applied on GPU
 *
 *  Input:
 *  	seed: int, used to shuffle shot gather, for quick debugging
 *  	offmin: float, minimum offset used for RTM
 *  	offmax: float, maximum offset used for RTM
 *
 *  Output:
 *
 *  Author: Yinbin Ma
 *  First created: 2018-01-23
 *  Last modified: 2018-01-23
 */

int main (int argc, char *argv[]) {
	initpar(argc, argv);

	// parameters related to machine
	int nt = 1500, noff = 175;

	// import parameter from command line
	// set up axis
	axis ax_t(nt, 0, 0.004, "time", "ms");
	axis ax_f(noff, 0, 1, "offset", "id");
	hyperCube dataCube({ax_t, ax_f});

	// read in data
	float *dataBaseRe  = new float[nt * noff];
	float *dataBaseIm  = new float[nt * noff];
	float *dataBaseAbs = new float[nt * noff];
	float *dataBaseAng = new float[nt * noff];
	float *dataMonRe   = new float[nt * noff];
	float *dataMonIm   = new float[nt * noff];
	float *dataMonAbs  = new float[nt * noff];
	float *dataMonAng  = new float[nt * noff];
	sreed("dataB", dataBaseRe, sizeof(float) * nt * noff);
	sreed("dataM", dataMonRe, sizeof(float) * nt * noff);
	float *tmpR = new float[nt];
	float *tmpI = new float[nt];
	float eps = 1000.0;
	for (int ioff = 0; ioff < noff; ioff++) {
		cout << "work on offset " << ioff << ", out of " << noff << endl;
		hilbert(&dataBaseRe[ioff * nt], &dataBaseIm[ioff * nt], nt);
		hilbert(&dataMonRe[ioff * nt], &dataMonIm[ioff * nt], nt);
		memcpy(tmpR, &dataBaseRe[ioff * nt], sizeof(float) * nt);
		memcpy(tmpI, &dataBaseIm[ioff * nt], sizeof(float) * nt);
		fftHarmonic(tmpR, tmpI, &dataBaseRe[ioff * nt], &dataBaseIm[ioff * nt], nt);
		memcpy(tmpR, &dataMonRe[ioff * nt], sizeof(float) * nt);
		memcpy(tmpI, &dataMonIm[ioff * nt], sizeof(float) * nt);
		fftHarmonic(tmpR, tmpI, &dataMonRe[ioff * nt], &dataMonIm[ioff * nt], nt);
		float energyB;
		if (ioff == 0) energyB = energy2D(&dataBaseRe[ioff * nt], nt, 1);
		float energyM;
		if (ioff == 0) energyM = energy2D(&dataMonRe[ioff * nt], nt, 1);
		for (int it = 0, i = ioff * nt; it < nt; it++, i++) {
			dataBaseAbs[i] = sqrt(dataBaseRe[i] * dataBaseRe[i] + dataBaseIm[i] * dataBaseIm[i]);
			dataMonAbs[i]  = sqrt(dataMonRe[i] * dataMonRe[i]   + dataMonIm[i] * dataMonIm[i]);
			dataBaseAng[i] = asin(dataBaseRe[i] / dataBaseAbs[i]);
			dataMonAng[i]  = asin(dataMonRe[i] / dataMonAbs[i]);
			dataBaseAng[i] *= dataMonAbs[i] * dataMonAbs[i];
			dataMonAng[i] *= dataMonAbs[i] * dataMonAbs[i];
		}
	}
	srite("angB", dataBaseAng, sizeof(float) * nt * noff);
	dataCube.writeAxes("angB");
	srite("angM", dataMonAng, sizeof(float) * nt * noff);
	dataCube.writeAxes("angM");

	return 0;
}
