#ifndef UTILGENESIS_H
#define UTILGENESIS_H 1
#include "SEPMYinc.h"
#include "Vector.h"

void generateRicker(float *wavelet, float fund, float dt, int nt, float tdelay);
void extractPar(SEPMY::hyperCube *cube, int &n, float &o, float &d, int index);

SEPMY::hyperCube expandModelCube(SEPMY::hyperCube &small, int BLOCK_SIZE = 16, int FAT = 4);
void fillModel(SEPMY::hyperCube &small, SEPMY::hyperCube &large, float *smallval, float *largeval);
SEPMY::hyperCube buildImageCube(SEPMY::hyperCube &modelCube, int nhx, int ns);
SEPMY::hyperCube buildGradCube(SEPMY::hyperCube &modelCube, int nhx, int ns);


void generateRicker(float * wavelet, float fund, float dt, int nt, float tdelay) {
	float PI = 3.1415926;
    for (int it = 0; it < nt; it++) {
    	double t = (dt * it - tdelay) * 0.001;
        double tmp0 = PI * PI * fund * fund * t * t;
        wavelet[it] = (1.0 - 2 * tmp0) * exp(-tmp0);
    }
}

void extractPar(SEPMY::hyperCube *cube, int &n, float &o, float &d, int index) {
	SEPMY::axis ax = cube->getAxis(index);
	n = ax.n;
	o = ax.o;
	d = ax.d;
	std::cout << "  n = " << n << ", o = " << o << ", d = " << d << std::endl;
}

SEPMY::hyperCube expandModelCube(SEPMY::hyperCube &small, int BLOCK_SIZE, int FAT) {
	std::vector<SEPMY::axis> ax1 = small.getAxes();
	vector<SEPMY::axis> ax2;
	for (int i = ax1.size() - 1; i >= 0; i--) {
		SEPMY::axis ax = ax1[i];
		int n = ax.n;
		int nnew = ((n + BLOCK_SIZE - 1 + 150) / BLOCK_SIZE) * BLOCK_SIZE + FAT * 2;
		ax.n = nnew;
		int shift = (nnew - n) / 2;
		ax.o = ax1[i].o - ax1[i].d * shift;
		ax2.push_back(ax);
	}
	SEPMY::hyperCube res(ax2);
	return res;
}


void fillModel(SEPMY::hyperCube &small, SEPMY::hyperCube &large, float *smallval, float *largeval) {
	std::vector<SEPMY::axis> ax1 = small.getAxes();
	std::vector<SEPMY::axis> ax2 = large.getAxes();
    int nz1 = ax1[0].n;
    int nx1 = ax1[1].n;
    int nz2 = ax2[1].n;
    int nx2 = ax2[0].n;
    int shiftX = std::round((ax1[0].o - ax2[1].o) / ax1[0].d);
    int shiftZ = std::round((ax1[1].o - ax2[0].o) / ax1[1].d);
    std::cout << "shiftX = " << shiftX << ", shiftZ = " << shiftZ << std::endl;
	for (int iz2 = 0; iz2 < nz2; iz2++) {
		for (int ix2 = 0; ix2 < nx2; ix2++) {
    		int iz1 = std::min(max(iz2 - shiftZ, 0), nz1 - 1);
    		int ix1 = std::min(max(ix2 - shiftX, 0), nx1 - 1);
    		largeval[iz2 * nx2 + ix2] = smallval[ix1 * nz1 + iz1];
    	}
    }
}

SEPMY::hyperCube buildImageCube(SEPMY::hyperCube &modelCube, int nhx, int ns) {
	std::vector<SEPMY::axis> ax = modelCube.getAxes();
	int dx = ax[0].d;
	int SP = 3;
  	SEPMY::axis ahx(nhx, -SP * dx * (nhx / 2), SP * dx, "nhx", "meter");
  	SEPMY::axis as(ns, 0., 1, "shot", "number");
  	ax.push_back(ahx);
  	ax.push_back(as);
  	SEPMY::hyperCube res(ax);
  	return res;
}

SEPMY::hyperCube buildGradCube(SEPMY::hyperCube &modelCube, int nhx, int ns) {
	std::vector<SEPMY::axis> ax = modelCube.getAxes();
	int dx = ax[0].d;
	int SP = 3;
  	SEPMY::axis as(ns, 0., 1, "shot", "number");
  	ax.push_back(as);
  	SEPMY::hyperCube res(ax);
  	return res;
}
#endif
