#include "cuda_runtime.h"
#include "device_launch_parameters.h"

typedef long long llong;
#define THICKNESS 50
#define PI 3.1415926535
#define BLOCKZ_SIZE 16
#define BLOCKX_SIZE 16
#define BLOCKY_SIZE 16
#define BLOCK_SIZE 16
#define FAT 4
#define SP 3
#define COEFFS_SIZE 13
#define radius 4

#define COEF0 -2.8472222222222222
#define COEF1 1.6
#define COEF2 -0.2
#define COEF3 0.025396825396825397
#define COEF4 -0.0017857142857142857
#define FAT 4

__constant__ int ntGPU;
__constant__ int nzGPU;
__constant__ int nxGPU;
__constant__ int nsGPU;
__constant__ int nrGPU;
__constant__ int tsamplegpu;
__constant__ int modelSizegpu;
__constant__ int waveSizegpu;
__constant__ float dtgpu;
__constant__ float dxgpu;
__constant__ int shift1GPU;
__constant__ int shift2GPU;
__constant__ int shift3GPU;
__constant__ int shift4GPU;
__constant__ int nsincGPU;
__constant__ float * sinc_s_table;
__constant__ float * sinc_d_table;
__constant__ int *srcgeom_gpu;
__constant__ float *sourceGPU;
__constant__ int *sourceLocGPU;
__constant__ float *sourceWeightGPU;
__constant__ float *dataShotGPU;
__constant__ float *dataRecGPU;
__constant__ int *dataLocGPU;
__constant__ float *dataWeightGPU;
__constant__ float *velDt2GPU;
__constant__ float *velDtDxGPU;
__constant__ float *epsilonTermGPU;
__constant__ float *deltaTermGPU;
__constant__ float *imageGPU;
__constant__ float *tmpDataGPU;
__constant__ int hx0GPU;
__constant__ int countGPU;
__constant__ int countSGPU;
/*
  Created by Yinbin Ma
  Time marching operator Unext = F(Ucur, Uprev) with taper/source injection.
  All the computation must be done here, to be changecd to GPU
  Use carefully and have fun.

  First Created: 2015-10-30
  Last Modified: 2015-10-30

*/

extern "C" __global__ void test(){
    printf("Hi Cuda World");
}

extern "C"  __global__ void marchingKernelGPU(float *prev, float *curr, float *next) {
	// marching
	int indexX = blockIdx.x * blockDim.x + threadIdx.x;
	int indexZ = blockIdx.z * blockDim.z + threadIdx.z;
	int index = (indexZ + FAT) * nxGPU + indexX + FAT;
	int indexH = index;
	int indexV = index + nxGPU * nzGPU;
	//if (blockIdx.x == 0 && blockIdx.z == 0) printf("inside marching kernel\n");
	float tempH = COEF0 * curr[indexH]
			    + COEF1 * (curr[indexH - 1] + curr[indexH + 1])
				+ COEF2 * (curr[indexH - 2] + curr[indexH + 2])
				+ COEF3 * (curr[indexH - 3] + curr[indexH + 3])
				+ COEF4 * (curr[indexH - 4] + curr[indexH + 4]);
	float tempV = COEF0 * curr[indexV]
				+ COEF1 * (curr[indexV - nxGPU] + curr[indexV + nxGPU])
				+ COEF2 * (curr[indexV - 2 * nxGPU] + curr[indexV + 2 * nxGPU])
				+ COEF3 * (curr[indexV - 3 * nxGPU] + curr[indexV + 3 * nxGPU])
				+ COEF4 * (curr[indexV - 4 * nxGPU] + curr[indexV + 4 * nxGPU]);

	next[indexH] = epsilonTermGPU[index] * tempH
                 + deltaTermGPU[index] * tempV
	             + curr[indexH] + curr[indexH] - prev[indexH];
	next[indexV] = velDtDxGPU[index] * tempV
                 + deltaTermGPU[index] * tempH
	             + curr[indexV] + curr[indexV] - prev[indexV];
}



extern "C" __global__ void injectWaveletGPU(int it, int isinc, float *p){
	  int id = blockIdx.x * blockDim.x + threadIdx.x;

	  if(id < countSGPU) {
		  float tmp = sinc_s_table[isinc * nsincGPU + 0] * sourceGPU[it - 3] * 1
            + sinc_s_table[isinc * nsincGPU + 1] * sourceGPU[id * (2 * ntGPU + 1) + it - 2] * 1
            + sinc_s_table[isinc * nsincGPU + 2] * sourceGPU[id * (2 * ntGPU + 1) + it - 1] * 1
            + sinc_s_table[isinc * nsincGPU + 3] * sourceGPU[id * (2 * ntGPU + 1) + it + 0] * 1
            + sinc_s_table[isinc * nsincGPU + 4] * sourceGPU[id * (2 * ntGPU + 1) + it + 1] * 1
            + sinc_s_table[isinc * nsincGPU + 5] * sourceGPU[id * (2 * ntGPU + 1) + it + 2] * 1
            + sinc_s_table[isinc * nsincGPU + 6] * sourceGPU[id * (2 * ntGPU + 1) + it + 3] * 1
            + sinc_s_table[isinc * nsincGPU + 7] * sourceGPU[id * (2 * ntGPU + 1) + it + 4] * 1;
		    //printf("id = %d, nsGPU = %d, countSGPU = %d, energy = %f\n", id, nsGPU, countSGPU, tmp);
		    int loc = sourceLocGPU[id];
		    p[loc] =  p[loc] + tmp;
		    p[loc + nzGPU * nxGPU] =  p[loc + nzGPU * nxGPU] + tmp;
	  }

  //printf("tmp = %f \n", tmp);
  /*
  if (it == ntGPU && isinc == 0 && ix == 1) {
		for (int i = 0; i < 4; i++) {
		    for (int j = 0; j < nsincGPU; j++) {
		    	printf("sample = %d, j = %d, coef = %f \n", i, j, sinc_s_table[i * nsincGPU + j]);
		    }
		}
  }
  */

}



//Similar to src_inject, but ensures injections only occurs within the receiver plane
extern "C" __global__ void injectDataKernelGPU(int it, int isinc,float *p){
  //printf("what the fuck\n");
  int ir = blockIdx.x * blockDim.x + threadIdx.x;
  int fracR = blockIdx.z * blockDim.z + threadIdx.z;
  int id = ir + fracR * nrGPU;
  //if (ir == 100) printf("id = %d, count = %d \n", id, countGPU);

  if(id < countGPU){
	  float tmp = sinc_s_table[isinc * nsincGPU]     * dataShotGPU[ntGPU * id + it - 3]
	            + sinc_s_table[isinc * nsincGPU + 1] * dataShotGPU[ntGPU * id + it - 2]
	            + sinc_s_table[isinc * nsincGPU + 2] * dataShotGPU[ntGPU * id + it - 1]
	            + sinc_s_table[isinc * nsincGPU + 3] * dataShotGPU[ntGPU * id + it + 0]
	            + sinc_s_table[isinc * nsincGPU + 4] * dataShotGPU[ntGPU * id + it + 1]
	            + sinc_s_table[isinc * nsincGPU + 5] * dataShotGPU[ntGPU * id + it + 2]
	            + sinc_s_table[isinc * nsincGPU + 6] * dataShotGPU[ntGPU * id + it + 3]
	            + sinc_s_table[isinc * nsincGPU + 7] * dataShotGPU[ntGPU * id + it + 4];
	  int loc = dataLocGPU[id];
	  p[loc] += tmp;// + 1;
	  p[loc + nxGPU * nzGPU] += tmp;// + 1;
	  //if (id == 250) {
	  //	printf("id = %d, ix = %d, iz = %d, value = %f \n", id, loc % nxGPU, loc / nxGPU, dataShotGPU[ntGPU * id + it]);
	  //}
  }

}

//Similar to src_inject, but ensures injections only occurs within the receiver plane
extern "C" __global__ void extractDataKernelGPU(int it, int isinc,float *p){
  //printf("what the fuck\n");
  int ir = blockIdx.x * blockDim.x + threadIdx.x;
  int fracR = blockIdx.z * blockDim.z + threadIdx.z;
  int id = ir + fracR * nrGPU;
  //if (ir == 100) printf("id = %d, count = %d \n", id, countGPU);

  if(id < countGPU){
	  float tmp = sinc_s_table[isinc * nsincGPU]     * dataShotGPU[ntGPU * id + it - 3]
	            + sinc_s_table[isinc * nsincGPU + 1] * dataShotGPU[ntGPU * id + it - 2]
	            + sinc_s_table[isinc * nsincGPU + 2] * dataShotGPU[ntGPU * id + it - 1]
	            + sinc_s_table[isinc * nsincGPU + 3] * dataShotGPU[ntGPU * id + it + 0]
	            + sinc_s_table[isinc * nsincGPU + 4] * dataShotGPU[ntGPU * id + it + 1]
	            + sinc_s_table[isinc * nsincGPU + 5] * dataShotGPU[ntGPU * id + it + 2]
	            + sinc_s_table[isinc * nsincGPU + 6] * dataShotGPU[ntGPU * id + it + 3]
	            + sinc_s_table[isinc * nsincGPU + 7] * dataShotGPU[ntGPU * id + it + 4];
	  int loc = dataLocGPU[id];
//	  p[loc] += tmp;// + 1;
//	  p[loc + nxGPU * nzGPU] += tmp;// + 1;
	  dataShotGPU[ntGPU * id + it] += p[loc] + p[loc + nxGPU * nzGPU];

	  //if (id == 250) {
	  //	printf("id = %d, ix = %d, iz = %d, value = %f \n", id, loc % nxGPU, loc / nxGPU, dataShotGPU[ntGPU * id + it]);
	  //}
  }

}

// imaging kernel
extern "C" __global__ void img_hx_kernel(float *img, float *prev, float *curr, float *next, float *currR, int ihx) {
	int indexX = blockIdx.x * blockDim.x + threadIdx.x;
	int indexZ = blockIdx.z * blockDim.z + threadIdx.z;
	int ihxshift = ihx + hx0GPU;
	if (indexX > ihxshift * SP && indexX < nxGPU - ihxshift * SP) {
		int index = (indexZ + FAT) * nxGPU + indexX + FAT;
		int indexH = index;
		int indexV = index + nxGPU * nzGPU;
		int imgLoc = (ihx + hx0GPU) * nzGPU * nxGPU + indexH;
		img[imgLoc] += (prev[indexH - ihx * SP] - 2.0 * curr[indexH - ihx * SP] + next[indexH - ihx * SP]) * currR[indexH + ihx * SP]; // * indexZ * indexZ;
		img[imgLoc] += (prev[indexV - ihx * SP] - 2.0 * curr[indexV - ihx * SP] + next[indexV - ihx * SP]) * currR[indexV + ihx * SP]; // * indexZ * indexZ;
		//img[imgLoc] += curr[indexH - ihx * SP] * currR[indexH + ihx * SP]; // * indexZ * indexZ;
		//img[imgLoc] += curr[indexV - ihx * SP] * currR[indexV + ihx * SP]; // * indexZ * indexZ;

	}
	//if (img[imgLoc] != 0) {
	 //  printf("index = %d, value = %f \n", index, img[imgLoc]);
	//}
}


// imaging kernel
extern "C" __global__ void grad_kernel(float *velGrad, float *epsilonGrad, float *deltaGrad, float *prev, float *curr, float *next, float *currR) {
	int indexX = blockIdx.x * blockDim.x + threadIdx.x;
	int indexZ = blockIdx.z * blockDim.z + threadIdx.z;
	if (1) {
		int index = (indexZ + FAT) * nxGPU + indexX + FAT;
		int indexH = index;
		int indexV = index + nxGPU * nzGPU;
		int imgLoc = indexH;

		float partX = COEF0 * curr[indexH]
				    + COEF1 * (curr[indexH - 1] + curr[indexH + 1])
					+ COEF2 * (curr[indexH - 2] + curr[indexH + 2])
					+ COEF3 * (curr[indexH - 3] + curr[indexH + 3])
					+ COEF4 * (curr[indexH - 4] + curr[indexH + 4]);
		float partZ = COEF0 * curr[indexV]
					+ COEF1 * (curr[indexV - nxGPU] + curr[indexV + nxGPU])
					+ COEF2 * (curr[indexV - 2 * nxGPU] + curr[indexV + 2 * nxGPU])
					+ COEF3 * (curr[indexV - 3 * nxGPU] + curr[indexV + 3 * nxGPU])
					+ COEF4 * (curr[indexV - 4 * nxGPU] + curr[indexV + 4 * nxGPU]);

		velGrad[imgLoc]     += (prev[indexH] - 2.0 * curr[indexH] + next[indexH]) * currR[indexH];
		velGrad[imgLoc]     += (prev[indexV] - 2.0 * curr[indexV] + next[indexV]) * currR[indexV];
		epsilonGrad[imgLoc] += partX * currR[indexH];
		deltaGrad[imgLoc] += partX * currR[indexV] + partZ * currR[indexH];
	}

}


extern "C"  __global__ void taperGPU(float *wave, float* wave2, float *taper) {
	int index1 = blockIdx.x * blockDim.x + threadIdx.x;
	int index2 = blockIdx.z * blockDim.z + threadIdx.z;
	int index = (index2 + FAT) * nxGPU + index1 + FAT;
	int indexH = index;
	int indexV = index + nxGPU * nzGPU;
	wave[indexH] = wave[indexH] * taper[indexH];
	wave[indexV] = wave[indexV] * taper[indexH];
	wave2[indexH] = wave2[indexH] * taper[indexH];
	wave2[indexV] = wave2[indexV] * taper[indexH];
}


